import logging

from aenum import Enum, NoAlias

_EVENT_PROPERTIES = ["label.category", "event", "label.description", "level"]
_EVENT_MEMBER_VALUE = {"description": 0, "level": 1}
_DEFAULT_LOG_LEVEL = logging.INFO


class Label(Enum):
    """Базовый класс для создания классов событий.

    Структура логируемых событий, состоит из:
        - имени категории события (category),
        - имени события (event),
        - описания события (description) в свободной форме
        - уровня логирования (level) (по умолчанию используетя уровень INFO).

    Данные могуть быть представлены как словарь:
    {"label.category": "SomeClass", "label.event": "TEST_EVENT", "label.description": "описание", "level": 20}.
    """

    _settings_ = NoAlias

    @property
    def event_properties(self):
        """Можно переопределить в дочерних классах, чтобы изменить набор возвращаемых ключей."""
        return _EVENT_PROPERTIES

    @property
    def category(self):
        return self.__class__.__name__

    @property
    def event(self):
        return str(self)

    @property
    def description(self):
        if isinstance(self._value_, tuple):
            desc = str(self._value_[_EVENT_MEMBER_VALUE["description"]])
        else:
            desc = str(self._value_)

        return desc

    @property
    def level(self):
        level = _DEFAULT_LOG_LEVEL
        if isinstance(self._value_, tuple):
            try:
                level = self._value_[_EVENT_MEMBER_VALUE["level"]]
            except IndexError:
                level = _DEFAULT_LOG_LEVEL

        return level

    def keys(self):
        """Позволяет распаковывать события как словарь"""
        return self.event_properties

    def __str__(self):
        return self._name_

    def __repr__(self):
        return self.__str__()

    def __getitem__(self, key):
        """Позволяет распаковывать события как словарь"""
        if key not in self.event_properties:
            raise KeyError
        key = key.replace('label.', '')
        return getattr(self, key)




import logging
from collections import OrderedDict
from logging import LogRecord

from pythonjsonlogger import jsonlogger


labelgun_logger = logging.getLogger(__name__)


class StructlogJsonFormatter(jsonlogger.JsonFormatter):
    """Formatter для стандартного python логгера logging. Позволяет конфигурировать себя при помощи списка
    процессоров от structlog и выполняет логирование в json формате."""

    _cache_loggers = {}

    def __init__(self, fmt=None, *args, proc=tuple(), **kwargs):
        super().__init__(fmt, *args, **kwargs)
        self._structlog_processors = proc

    def set_structlog_processors(self, proc: list):
        self._structlog_processors = proc

    def _get_logger(self, name: str):
        log = StructlogJsonFormatter._cache_loggers.get(name)

        if log is None:
            log = logging.getLogger(name)
            StructlogJsonFormatter._cache_loggers[name] = log

        return log

    def add_fields(self, log_record: OrderedDict, record: LogRecord, message_dict: dict):
        """Выполяет преобразование стандартного сообщения в формат structlog"""
        super().add_fields(log_record, record, message_dict)

        msg = log_record.pop('message', '')
        event_dict = {'event': msg}

        if isinstance(record.msg, str):
            log_record['event'] = msg
            event_dict = log_record

        logger = self._get_logger(record.name)
        level = record.levelname.lower()

        for proc in self._structlog_processors:
            try:
                event_dict = proc(logger, level, event_dict)
            except Exception:
                labelgun_logger.exception("STRUCTLOG_PROCESSOR_THREW_AN_EXCEPTION")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['labelgun', 'labelgun.integrations']

package_data = \
{'': ['*']}

install_requires = \
['aenum>=3.0.0,<4.0.0']

extras_require = \
{'logger': ['python-json-logger>=2.0,<2.1', 'structlog>=20,<22']}

setup_kwargs = {
    'name': 'labelgun',
    'version': '0.2.1',
    'description': 'Library to define system events',
    'long_description': 'None',
    'author': 'Aleksey Petrunnik',
    'author_email': 'apetrunnik@usetech.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import logging


def dict_msg_processor(logger: logging.Logger, name: str, event_dict: dict):
    """Позволяет переложить конвертацию данных в строку на Formatter библиотеки logging. Данная возможность полезна при
    внедрении structlog в приложение, где часть логов уже пишется при помощи библиотеки logging.

    P.S. Иными словами, данный процессор служит заменой для structlog.dev.ConsoleRenderer или
    structlog.processors.JSONRenderer и позволяет сказать библиотеке structlog, что данные, которые необходимо
    залогировать сформированы и можно передать их дальше.

    Данный процессор должен быть ВСЕГДА ПОСЛЕДНИМ в списке процессоров.

    """
    return (event_dict,), {}


def convert_event_dict_to_str_processor(logger: logging.Logger, name: str, event_dict: dict):
    """Выполняет конвертацию всех данных в строки.

    Данный процессор помогает избежать проблемы, если логи вашего приложения загружаются в elasticsearch для анализа.

    Пример решаемой проблемы:
    Логи с нескольких сервисов собираются в одном индексе elasticsearch и каждый из этих сервисов в логах использует
    переменную с одинаковым именем, но содержание у них разное. На пример в одном сервисе логируется user_id=1, а в
    другом user_id='4v43gbv33'. В этом случае в elasticsearch попадут логи только того сервиса, сообщение от которого
    будет обработано раньше. Это происходит потому, что индекс в elasticsearch типизирован и одно поле не может хранить
    данные различных типов.

    """
    for param_key, param_value in event_dict.items():
        # не нужно приводить None к строке "None", это вызовет больше проблем.
        if param_value is None:
            continue

        event_dict[param_key] = str(param_value)

    return event_dict
